#define __AVR_ATmega328P__
/**
 * Digital Level Using LIS3LV02DL
 * Kerry Wong
 * http://www.kerrywong.com
 */

/** <editor-fold desc="Includes"> */
#include <binary.h>
#include <HardwareSerial.h>
#include <pins_arduino.h>
#include <WConstants.h>
#include <wiring.h>
#include <wiring_private.h>
#include <WProgram.h>
#include <EEPROM/EEPROM.h>
#include <LiquidCrystal/LiquidCrystal.h>
#include <Wire/Wire.h>
#include <Wire/utility/twi.h>
#include <avr/interrupt.h>
/** </editor-fold> */

/** <editor-fold desc="Pin Definitions"> */
const int PIN_DATAOUT = 11; //MOSI lis3lv02dl pin3
const int PIN_DATAIN = 12; //MISO lis3lv02dl pin2
const int PIN_SPICLOCK = 13;//sck lis3lv02dl pin5
const int PIN_CS = 10;//ss lis3lv02dl pin6, active low

const int PIN_LCD_RS = 2;
const int PIN_LCD_EN = 3;
const int PIN_LCD_D4 = 7;
const int PIN_LCD_D5 = 6;
const int PIN_LCD_D6 = 5;
const int PIN_LCD_D7 = 4;

const int PIN_LCD_BACKLIGHT = 8;

const int BTN_PINS[] = {9, 14, 15};
const int BTN_BACKLIGHT = 0;
const int BTN_XYCAL = 1;
const int BTN_DISP_MODE = 2;
/** </editor-fold> */

/** <editor-fold desc="lis3lv02dl Register Addresses"> */
const int REG_WHO_AM_I = 0x0f; //3A
const int REG_OFFSET_X = 0x16;
const int REG_OFFSET_Y = 0x17;
const int REG_OFFSET_Z = 0x18;
const int REG_GAIN_X = 0x19;
const int REG_GAIN_Y = 0x1A;
const int REG_GAIN_Z = 0x1B;
const int REG_CTRL_REG1 = 0x20;
const int REG_CTRL_REG2 = 0x21;
const int REG_CTRL_REG3 = 0x22;
const int REG_HP_FILTER_RESET = 0x23;
const int REG_STATUS_REG = 0x27;
const int REG_OUTX_L = 0x28;
const int REG_OUTX_H = 0x29;
const int REG_OUTY_L = 0x2a;
const int REG_OUTY_H = 0x2b;
const int REG_OUTZ_L = 0x2c;
const int REG_OUTZ_H = 0x2d;
const int REG_FF_WU_CFG = 0x30;
const int REG_FF_WU_SRC = 0x31;
const int REG_FF_WU_ACK = 0x32;
const int REG_FF_WU_THS_L = 0x34;
const int REG_FF_WU_THS_H = 0x35;
const int REG_FF_WU_DURATION = 0x36;
const int REG_DD_CFG = 0x38;
const int REG_DD_SRC = 0x39;
const int REG_DD_ACK = 0x3a;
const int REG_DD_THSI_L = 0x3c;
const int REG_DD_THSI_H = 0x3d;
const int REG_DD_THSE_L = 0x3e;
const int REG_DD_THSE_H = 0x3f;
/** </editor-fold> */

/** <editor-fold desc="Axis Definitions"> */
const int AXIS_X = 1;
const int AXIS_Y = 2;
const int AXIS_Z = 3;

const int X_SIGN = 1;
const int Y_SIGN = 1;
const int Z_SIGN = -1;
/** </editor-fold> */

const int CTRL_REG2_FS = 7;
boolean btnPressed[] = {false, false, false};
LiquidCrystal lcd(PIN_LCD_RS, PIN_LCD_EN, PIN_LCD_D4, PIN_LCD_D5, PIN_LCD_D6, PIN_LCD_D7);
char dispLine1[] = "  X    Y    Z 2g";
char dispLine2[] = "  X    Y    Z 6g";

char dispFormatStr[] = "%5d%5d%5d";

int X_OFFSET = 0;
int Y_OFFSET = 0;
int Z_OFFSET = 0;

int frameCounter = 0;
int frameDelay = 5000;

int vx = 0, vy = 0, vz = 0;

boolean backLightOn = false;
boolean highGOn = false;

void delayMilliseconds(int ms) {
    for (int i = 0; i < ms; i++) {
        delayMicroseconds(1000);
    }
}

/** detects whether the ith button is pressed */
boolean buttonPressed(int btnIdx) {
    if (digitalRead(BTN_PINS[btnIdx]) == LOW && !btnPressed[btnIdx]) {
        delayMilliseconds(20);
        if (digitalRead(BTN_PINS[btnIdx]) == LOW) {
            btnPressed[btnIdx] = true;
            return true;
        }
    }

    return false;
}

/** detects whether the ith button is released */
boolean buttonRleased(int btnIdx) {
    if (digitalRead(BTN_PINS[btnIdx]) == HIGH && btnPressed[btnIdx]) {
        delayMilliseconds(20);
        if (digitalRead(BTN_PINS[btnIdx]) == HIGH) {
            btnPressed[btnIdx] = false;
            return true;
        }
    }
    return false;
}

//transfer a byte using SPI

byte spiTransfer(volatile byte data) {
    SPDR = data;

    while (!(SPSR & _BV(SPIF)));

    return SPDR;
}

byte readRegister(byte reg) {
    digitalWrite(PIN_CS, LOW);
    spiTransfer(reg | 0x80);
    byte b = spiTransfer(0);
    digitalWrite(PIN_CS, HIGH);

    return b;
}

void writeRegister(byte reg, byte data) {
    digitalWrite(PIN_CS, LOW);
    spiTransfer(reg & 0x7F);
    spiTransfer(data);
    digitalWrite(PIN_CS, HIGH);
}

void pinSetup() {
    pinMode(PIN_DATAOUT, OUTPUT);
    pinMode(PIN_DATAIN, INPUT);
    pinMode(PIN_SPICLOCK, OUTPUT);
    pinMode(PIN_CS, OUTPUT);

    pinMode(PIN_LCD_BACKLIGHT, OUTPUT);

    for (int i = 0; i < 3; i++) {
        pinMode(BTN_PINS[i], INPUT);
        digitalWrite(BTN_PINS[i], HIGH);
    }
}

void initSys() {
    //disable device
    digitalWrite(PIN_CS, HIGH);

    //initialize SPI
    SPCR = _BV(SPE) | _BV(MSTR) | _BV(CPOL) | _BV(CPHA);

    //enable all axes
    writeRegister(REG_CTRL_REG1, 0x87);
}

void setScale(byte s) {
    byte r = readRegister(REG_CTRL_REG2);

    if (s == 0) { // 2g
        r &= ~_BV(CTRL_REG2_FS);
    } else { //6g
        r |= _BV(CTRL_REG2_FS);
    }

    writeRegister(REG_CTRL_REG2, r);
}

void setup() {
    byte b = 0;

    //Serial.begin(9600);

    pinSetup();
    initSys();
    b = readRegister(REG_WHO_AM_I);
    //Serial.println(b, HEX); //should be 3A

    lcd.begin(16, 2);
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(dispLine1);
    setScale(0);
}

int getValue(int axis) {
    int r = 0;
    byte h, l;

    switch (axis) {
        case AXIS_X:
            l = readRegister(REG_OUTX_L);
            h = readRegister(REG_OUTX_H);

            break;
        case AXIS_Y:
            l = readRegister(REG_OUTY_L);
            h = readRegister(REG_OUTY_H);

            break;
        case AXIS_Z:
            l = readRegister(REG_OUTZ_L);
            h = readRegister(REG_OUTZ_H);

            break;
        default:
            l = 0;
            h = 0;
            break;
    }

    r = h << 8 | l;

    return r;
}

void loop() {
    char s[20];

    if (buttonPressed(BTN_BACKLIGHT)) {
        backLightOn = !backLightOn;

        if (backLightOn) {
            digitalWrite(PIN_LCD_BACKLIGHT, HIGH);
        } else {
            digitalWrite(PIN_LCD_BACKLIGHT, LOW);
        }
    } else if (buttonPressed(BTN_XYCAL)) {
        X_OFFSET += vx;
        Y_OFFSET += vy;
    } else if (buttonPressed(BTN_DISP_MODE)) {
        highGOn = !highGOn;
        lcd.setCursor(0, 0);
        if (highGOn) {
            setScale(1);
            lcd.print(dispLine2);
        } else {
            setScale(0);
            lcd.print(dispLine1);
        }
    } else {
        frameCounter++;

        if (frameCounter > frameDelay) {
            frameCounter = 0;
            vx = getValue(AXIS_X) * X_SIGN - X_OFFSET;
            vy = getValue(AXIS_Y) * Y_SIGN - Y_OFFSET;
            vz = getValue(AXIS_Z) * Z_SIGN - Z_OFFSET;

            lcd.setCursor(0, 1);
            sprintf(s, dispFormatStr, vx, vy, vz);
            lcd.print(s);
        }
    }

    for (int i = 0; i < 3; i++) {
        if (buttonRleased(i)) {
            //the button is released, no action is necessary.
        }
    }
}

byte getGain(int axis) {
    byte r = 0;

    switch (axis) {
        case AXIS_X:
            r = readRegister(REG_GAIN_X);
            break;
        case AXIS_Y:
            r = readRegister(REG_GAIN_Y);
            break;
        case AXIS_Z:
            r = readRegister(REG_GAIN_Z);
            break;
        default:
            r = 0xff;
            break;
    }

    return r;
}

byte getOffset(int axis) {
    byte r = 0;

    switch (axis) {
        case AXIS_X:
            r = readRegister(REG_OFFSET_X);
            break;
        case AXIS_Y:
            r = readRegister(REG_OFFSET_Y);
            break;
        case AXIS_Z:
            r = readRegister(REG_OFFSET_Z);
            break;
        default:
            r = 0xff;
            break;
    }

    return r;
}




